/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
	      programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

#include "alphCRYPT.h"

char 
*alberti(char *org, char *key)
{

	int itra, itrb, itrc;
	char *hlpr, *dst, *bkp;
	char alb[26][26] =   { { 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z' },
													{ 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y' },
													{ 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x' },
													{ 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w' },
													{ 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v' },
													{ 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u' },
													{ 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's' },
                                       { 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o' },
													{ 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n' },
													{ 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm' },
													{ 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k' },
													{ 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j' },
													{ 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h' },
													{ 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g' },
													{ 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f' },
													{ 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd' },
													{ 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h' },
													{ 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p' },
													{ 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i', 'c' },
													{ 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't', 'i' },
													{ 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r', 't' },
													{ 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e', 'r' },
													{ 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b', 'e' },
													{ 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l', 'b' },
													{ 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a', 'l' },
													{ 'l', 'b', 'e', 'r', 't', 'i', 'c', 'p', 'h', 'd', 'f', 'g', 'h', 'j', 'k', 'm', 'n', 'o', 's', 'u', 'v', 'w', 'x', 'y', 'z', 'a' }
												};

	hlpr=(char *) calloc(strlen(org), sizeof(char));
	dst=(char *) calloc(strlen(org), sizeof(char));
	bkp=(char *) calloc(strlen(org), sizeof(char));

	for(itra=0, itrb=0; itra<strlen(org); itra++)
		if(isalpha(org[itra]))
			bkp[itrb++]=tolower(org[itra]);

	for(itra=0, itrb=0; itra<strlen(bkp); itra++)
	{
		if(itrb==strlen(key))
			itrb=0;
		hlpr[itra]=key[itrb++];
	}

	if(!al_dec)
		for(itra=0, itrb=0; itra<strlen(bkp); itra++)
			dst[itrb++]=alb[hlpr[itra]-97][bkp[itra]-97];
	else
	{
		for(itra=0, itrc=0; itra<strlen(bkp); itra++)
			for(itrb=0; itrb<26; itrb++)
				if(bkp[itra]==alb[hlpr[itra]-97][itrb]) {
					dst[itrc++]=itrb+97;
					break;
				}
	}

	bkp=(char *) calloc(strlen(org), sizeof(char));
	for(itra=0, itrb=0; itra<strlen(org)-1; itra++)
		if(!isalpha(org[itra]))
			bkp[itra]=org[itra];
		else
			if(isupper(org[itra]))
				bkp[itra]=toupper(dst[itrb++]);
			else
				bkp[itra]=dst[itrb++];

	if(org[strlen(org)-1]=='\n')
    strcat(bkp, "\n");

	return bkp;

}
